"""Midea local B8 device."""

import logging
from enum import IntEnum
from typing import Any

from midealocal.device import MideaDevice
from midealocal.devices.b8.const import (
    B8CleanMode,
    B8ControlType,
    B8DeviceAttributes,
    B8ErrorCanFixDescription,
    B8ErrorType,
    B8FanLevel,
    B8FunctionType,
    B8MopState,
    B8Moviment,
    B8Speed,
    B8WaterLevel,
    B8WorkMode,
    B8WorkStatus,
)
from midealocal.devices.b8.message import (
    MessageB8Response,
    MessageQuery,
    MessageSet,
    MessageSetCommand,
)

_LOGGER = logging.getLogger(__name__)


class MideaB8Device(MideaDevice):
    """Midea B8 device."""

    def __init__(
        self,
        name: str,
        device_id: int,
        ip_address: str,
        port: int,
        token: str,
        key: str,
        protocol: int,
        model: str,
        subtype: int,
        customize: str,  # noqa: ARG002
    ) -> None:
        """Initialize Midea B8 device."""
        super().__init__(
            name=name,
            device_id=device_id,
            device_type=0xB8,
            ip_address=ip_address,
            port=port,
            token=token,
            key=key,
            protocol=protocol,
            model=model,
            subtype=subtype,
            attributes={
                B8DeviceAttributes.WORK_STATUS: B8WorkStatus.NONE.name.lower(),
                B8DeviceAttributes.FUNCTION_TYPE: B8FunctionType.NONE.name.lower(),
                B8DeviceAttributes.CONTROL_TYPE: B8ControlType.NONE.name.lower(),
                B8DeviceAttributes.MOVE_DIRECTION: B8Moviment.NONE.name.lower(),
                B8DeviceAttributes.CLEAN_MODE: B8CleanMode.NONE.name.lower(),
                B8DeviceAttributes.FAN_LEVEL: B8FanLevel.OFF.name.lower(),
                B8DeviceAttributes.AREA: 0,
                B8DeviceAttributes.WATER_LEVEL: B8WaterLevel.OFF.name.lower(),
                B8DeviceAttributes.VOICE_VOLUME: 0,
                B8DeviceAttributes.MOP: B8MopState.OFF.name.lower(),
                B8DeviceAttributes.CARPET_SWITCH: False,
                B8DeviceAttributes.SPEED: B8Speed.HIGH.name.lower(),
                B8DeviceAttributes.HAVE_RESERVE_TASK: False,
                B8DeviceAttributes.BATTERY_PERCENT: 0,
                B8DeviceAttributes.WORK_TIME: 0,
                B8DeviceAttributes.UV_SWITCH: False,
                B8DeviceAttributes.WIFI_SWITCH: False,
                B8DeviceAttributes.VOICE_SWITCH: False,
                B8DeviceAttributes.COMMAND_SOURCE: False,
                B8DeviceAttributes.ERROR_TYPE: B8ErrorType.NO.name.lower(),
                B8DeviceAttributes.ERROR_DESC: B8ErrorCanFixDescription.NO.name.lower(),
                B8DeviceAttributes.DEVICE_ERROR: False,
                B8DeviceAttributes.BOARD_COMMUNICATION_ERROR: False,
                B8DeviceAttributes.LASER_SENSOR_SHELTER: False,
                B8DeviceAttributes.LASER_SENSOR_ERROR: False,
            },
        )

    def build_query(self) -> list[MessageQuery]:
        """Midea B8 device build query."""
        return [MessageQuery(self._protocol_version)]

    def process_message(self, msg: bytes) -> dict[str, Any]:
        """Midea B8 device process message."""
        message = MessageB8Response(msg)
        _LOGGER.debug("[%s] Received: %s", self.device_id, message)
        new_status = {}
        for status in self._attributes:
            if hasattr(message, str(status)):
                value = getattr(message, str(status))
                if isinstance(value, IntEnum):  # lowercase name for IntEnums
                    value = value.name.lower()
                self._attributes[status] = value
                new_status[str(status)] = self._attributes[status]
        return new_status

    def _gen_set_msg_default_values(self) -> MessageSet:
        msg = MessageSet(self._protocol_version)
        msg.clean_mode = B8CleanMode[
            self.attributes[B8DeviceAttributes.CLEAN_MODE].upper()
        ]
        msg.fan_level = B8FanLevel[
            self.attributes[B8DeviceAttributes.FAN_LEVEL].upper()
        ]
        msg.water_level = B8WaterLevel[
            self.attributes[B8DeviceAttributes.WATER_LEVEL].upper()
        ]
        msg.voice_volume = self.attributes[B8DeviceAttributes.VOICE_VOLUME]
        return msg

    def set_work_mode(self, work_mode: B8WorkMode) -> None:
        """Midea B8 device set work mode."""
        if work_mode == B8WorkMode.WORK:
            self.set_attribute(
                B8DeviceAttributes.CLEAN_MODE,
                self.attributes[B8DeviceAttributes.CLEAN_MODE],
            )
            return

        msg = MessageSetCommand(self._protocol_version, work_mode=work_mode)
        self.build_send(msg)

    def set_attribute(self, attr: str, value: bool | int | str) -> None:
        """Midea B8 device set attribute."""
        try:
            msg = self._gen_set_msg_default_values()
            if attr == B8DeviceAttributes.CLEAN_MODE:
                msg.clean_mode = B8CleanMode[str(value).upper()]
            elif attr == B8DeviceAttributes.FAN_LEVEL:
                msg.fan_level = B8FanLevel[str(value).upper()]
            elif attr == B8DeviceAttributes.WATER_LEVEL:
                msg.water_level = B8WaterLevel[str(value).upper()]
            elif attr == B8DeviceAttributes.VOICE_VOLUME:
                msg.voice_volume = int(value)

            if msg is not None:
                self.build_send(msg)
        except KeyError:
            _LOGGER.exception("Wrong value for attribute %s: %s", attr, value)


class MideaAppliance(MideaB8Device):
    """Midea B8 appliance."""



"""Midea local B8 message."""

from midealocal.devices.b8.const import (
    B8CleanMode,
    B8ControlType,
    B8DeviceAttributes,
    B8ErrorCanFixDescription,
    B8ErrorRebootDescription,
    B8ErrorType,
    B8ErrorWarningDescription,
    B8FanLevel,
    B8FunctionType,
    B8MopState,
    B8Moviment,
    B8Speed,
    B8StatusType,
    B8WaterLevel,
    B8WorkMode,
    B8WorkStatus,
)
from midealocal.message import (
    BodyType,
    BoolParser,
    IntEnumParser,
    IntParser,
    MessageBody,
    MessageRequest,
    MessageResponse,
    MessageType,
)


class MessageB8Base(MessageRequest):
    """B8 message base."""

    def __init__(
        self,
        protocol_version: int,
        message_type: int,
        body_type: int,
    ) -> None:
        """Initialize B8 message base."""
        super().__init__(
            device_type=0xB8,
            protocol_version=protocol_version,
            message_type=message_type,
            body_type=body_type,
        )

    @property
    def _body(self) -> bytearray:
        raise NotImplementedError


class MessageQuery(MessageB8Base):
    """B8 message query."""

    def __init__(self, protocol_version: int) -> None:
        """Initialize B8 message query."""
        super().__init__(
            protocol_version=protocol_version,
            message_type=MessageType.query,
            body_type=BodyType.X32,
        )

    @property
    def _body(self) -> bytearray:
        return bytearray([0x01])


class MessageSet(MessageB8Base):
    """B8 message set."""

    def __init__(self, protocol_version: int) -> None:
        """Initialize B8 message set."""
        super().__init__(
            protocol_version=protocol_version,
            message_type=MessageType.set,
            body_type=BodyType.X22,
        )
        self.clean_mode = B8CleanMode.AUTO
        self.fan_level = B8FanLevel.NORMAL
        self.water_level = B8WaterLevel.LOW
        self.voice_volume = 0
        self.zone_id = 0

    @property
    def _body(self) -> bytearray:
        return bytearray(
            [
                0x02,
                0x02,
                self.clean_mode,
                self.fan_level,
                self.water_level,
                self.voice_volume,
                self.zone_id,
            ]
            + [0x00] * 7,
        )


class MessageSetCommand(MessageB8Base):
    """B8 message set command."""

    def __init__(self, protocol_version: int, work_mode: B8WorkMode) -> None:
        """Initialize B8 message set command."""
        super().__init__(
            protocol_version=protocol_version,
            message_type=MessageType.set,
            body_type=BodyType.X22,
        )
        self.work_mode = work_mode

    @property
    def _body(self) -> bytearray:
        return bytearray(
            [
                self.work_mode,
                0x00,
                0x00,
            ],
        )


class MessageB8GenericBody(MessageBody):
    """B8 message generic body."""

    def __init__(self, body: bytearray, offset: int) -> None:
        """Initialize B8 message generic body."""
        super().__init__(body)
        self.parser_list.extend(
            [
                IntEnumParser[B8WorkStatus](
                    B8DeviceAttributes.WORK_STATUS,
                    1 + offset,
                    B8WorkStatus,
                ),
                IntEnumParser[B8FunctionType](
                    B8DeviceAttributes.FUNCTION_TYPE,
                    2 + offset,
                    B8FunctionType,
                ),
                IntEnumParser[B8ControlType](
                    B8DeviceAttributes.CONTROL_TYPE,
                    3 + offset,
                    B8ControlType,
                ),
                IntEnumParser[B8Moviment](
                    B8DeviceAttributes.MOVE_DIRECTION,
                    4 + offset,
                    B8Moviment,
                ),
                IntEnumParser[B8CleanMode](
                    B8DeviceAttributes.CLEAN_MODE,
                    5 + offset,
                    B8CleanMode,
                ),
                IntEnumParser[B8FanLevel](
                    B8DeviceAttributes.FAN_LEVEL,
                    6 + offset,
                    B8FanLevel,
                ),
                IntParser(B8DeviceAttributes.AREA, 7 + offset),
                IntEnumParser[B8WaterLevel](
                    B8DeviceAttributes.WATER_LEVEL,
                    8 + offset,
                    B8WaterLevel,
                ),
                IntParser(B8DeviceAttributes.VOICE_VOLUME, 9 + offset, max_value=100),
                BoolParser(
                    B8DeviceAttributes.HAVE_RESERVE_TASK,
                    10 + offset,
                ),
                IntParser(
                    B8DeviceAttributes.BATTERY_PERCENT,
                    11 + offset,
                    max_value=100,
                ),
                IntParser(B8DeviceAttributes.WORK_TIME, 12 + offset),
                BoolParser(B8DeviceAttributes.UV_SWITCH, 13 + offset, bit=0),
                BoolParser(B8DeviceAttributes.WIFI_SWITCH, 13 + offset, bit=1),
                BoolParser(B8DeviceAttributes.VOICE_SWITCH, 13 + offset, bit=2),
                BoolParser(B8DeviceAttributes.COMMAND_SOURCE, 13 + offset, bit=6),
                BoolParser(B8DeviceAttributes.DEVICE_ERROR, 13 + offset, bit=7),
                IntEnumParser[B8ErrorType](
                    B8DeviceAttributes.ERROR_TYPE,
                    14 + offset,
                    B8ErrorType,
                ),
                IntEnumParser[B8MopState](
                    B8DeviceAttributes.MOP,
                    16 + offset,
                    B8MopState,
                    default_value=B8MopState.LACK_WATER,
                ),
                BoolParser(B8DeviceAttributes.CARPET_SWITCH, 17 + offset),
                BoolParser(
                    B8DeviceAttributes.LASER_SENSOR_ERROR,
                    18 + offset,
                    bit=0,
                ),
                BoolParser(
                    B8DeviceAttributes.LASER_SENSOR_SHELTER,
                    18 + offset,
                    bit=1,
                ),
                BoolParser(
                    B8DeviceAttributes.BOARD_COMMUNICATION_ERROR,
                    18 + offset,
                    bit=2,
                ),
                IntEnumParser[B8Speed](B8DeviceAttributes.SPEED, 19 + offset, B8Speed),
            ],
        )
        self.parse_all()

        # Error description without parser
        self.error_desc: (
            B8ErrorCanFixDescription
            | B8ErrorRebootDescription
            | B8ErrorWarningDescription
        ) = B8ErrorCanFixDescription.NO
        error_type = getattr(self, B8DeviceAttributes.ERROR_TYPE, B8ErrorType.NO)
        if error_type == B8ErrorType.CAN_FIX:
            self.error_desc = B8ErrorCanFixDescription(
                self.read_byte(body, 15 + offset),
            )
        elif error_type == B8ErrorType.REBOOT:
            self.error_desc = B8ErrorRebootDescription(
                self.read_byte(body, 15 + offset),
            )
        elif error_type == B8ErrorType.WARNING:
            self.error_desc = B8ErrorWarningDescription(
                self.read_byte(body, 15 + offset),
            )


class MessageB8WorkStatusBody(MessageB8GenericBody):
    """B8 message work status body."""

    def __init__(self, body: bytearray) -> None:
        """Initialize B8 message work status body."""
        super().__init__(body, 1)


class MessageB8NotifyBody(MessageB8GenericBody):
    """B8 message notify body."""

    def __init__(self, body: bytearray) -> None:
        """Initialize B8 message notify body."""
        super().__init__(body, 0)


class MessageB8Response(MessageResponse):
    """B8 message response."""

    def __init__(self, message: bytes) -> None:
        """Initialize B8 message response."""
        super().__init__(bytearray(message))
        body = MessageB8Response.parse_body(
            MessageType(self.message_type),
            super().body,
        )
        if body is not None:
            self.set_body(body)
            self.set_attr()

    @staticmethod
    def parse_body(message_type: MessageType, body: bytearray) -> MessageBody | None:
        """Parse body."""
        body_type = body[0]
        status_type = body[1]
        if (
            message_type == MessageType.query
            and body_type == BodyType.X32
            and status_type == B8StatusType.X01
        ):
            return MessageB8WorkStatusBody(body)
        if message_type == MessageType.notify1 and body_type == BodyType.X42:
            return MessageB8NotifyBody(body)
        return None



"""Midea local B8 device const."""

from enum import IntEnum, StrEnum


class B8DeviceAttributes(StrEnum):
    """Midea B8 device attributes."""

    WORK_STATUS = "work_status"
    FUNCTION_TYPE = "function_type"
    CONTROL_TYPE = "control_type"
    MOVE_DIRECTION = "move_direction"
    CLEAN_MODE = "clean_mode"
    FAN_LEVEL = "fan_level"
    AREA = "area"
    WATER_LEVEL = "water_level"
    VOICE_VOLUME = "voice_volume"
    MOP = "mop"
    CARPET_SWITCH = "carpet_switch"
    SPEED = "speed"
    HAVE_RESERVE_TASK = "have_reserve_task"
    BATTERY_PERCENT = "battery_percent"
    WORK_TIME = "work_time"
    UV_SWITCH = "uv_switch"
    WIFI_SWITCH = "wifi_switch"
    VOICE_SWITCH = "voice_switch"
    COMMAND_SOURCE = "command_source"
    ERROR_TYPE = "error_type"
    ERROR_DESC = "error_desc"
    DEVICE_ERROR = "device_error"
    BOARD_COMMUNICATION_ERROR = "board_communication_error"
    LASER_SENSOR_SHELTER = "laser_sensor_shelter"
    LASER_SENSOR_ERROR = "laser_sensor_error"


class B8WorkMode(IntEnum):
    """Midea B8 work mode."""

    CHARGE = 0x01
    WORK = 0x02
    STOP = 0x03
    PAUSE = 0x1B


class B8WorkStatus(IntEnum):
    """Midea B8 work status."""

    NONE = 0x00
    CHARGE = 0x01
    WORK = 0x02
    STOP = 0x03
    CHARGING_ON_DOCK = 0x04
    RESERVE_TASK_FINISHED = 0x05
    CHARGE_FINISH = 0x06
    CHARGING_WITH_WIRE = 0x07
    PAUSE = 0x08
    UPDATING = 0x09
    SAVING_MAP = 0x0A
    ERROR = 0x0B
    SLEEP = 0x0C
    CHARGE_PAUSE = 0x0D
    RELOCATE = 0x0E
    ELECTROLYSED_WATER_MAKING = 0x0F
    DUST_COLLECTING = 0x10
    BACK_DUST_COLLECTING = 0x11
    SLEEP_IN_STATION = 0x12


class B8FunctionType(IntEnum):
    """Midea B8 function type."""

    NONE = 0x00
    DUST_BOX_CLEANING = 0x01
    WATER_TANK_CLEANING = 0x02


class B8ControlType(IntEnum):
    """Midea B8 control type."""

    NONE = 0x0
    MANUAL = 0x1
    AUTO = 0x2


class B8Moviment(IntEnum):
    """Midea B8 movement."""

    NONE = 0x0
    FORWARD = 0x1
    BACK = 0x2
    LEFT = 0x3
    RIGHT = 0x4


class B8CleanMode(IntEnum):
    """Midea B8 clean mode."""

    NONE = 0x00
    RANDOM = 0x01
    ARC = 0x02
    EDGE = 0x03
    EMPHASES = 0x04
    SCREW = 0x05
    BED = 0x06
    WIDE_SCREW = 0x07
    AUTO = 0x08
    AREA = 0x09
    ZONE_INDEX = 0x0A
    ZONE_RECT = 0x0B
    PATH = 0x0C


class B8FanLevel(IntEnum):
    """Midea B8 fan level."""

    OFF = 0x0
    SOFT = 0x1
    NORMAL = 0x2
    HIGH = 0x3
    LOW = 0x4


class B8WaterLevel(IntEnum):
    """Midea B8 water level."""

    OFF = 0x0
    LOW = 0x1
    NORMAL = 0x2
    HIGH = 0x3


class B8MopState(IntEnum):
    """Midea B8 mop state."""

    OFF = 0x0
    ON = 0x1
    LACK_WATER = 0x2


class B8Speed(IntEnum):
    """Midea B8 speed."""

    LOW = 0x1
    HIGH = 0x0


class B8ErrorType(IntEnum):
    """Midea B8 error type."""

    NO = 0x00
    CAN_FIX = 0x01
    REBOOT = 0x02
    WARNING = 0x03


class B8ErrorCanFixDescription(IntEnum):
    """Midea B8 error can fix description."""

    NO = 0x0
    FIX_DUST = 0x01
    FIX_WHEEL_HANG = 0x02
    FIX_WHEEL_OVERLOAD = 0x03
    FIX_SIDE_BRUSH_OVERLOAD = 0x04
    FIX_ROLL_BRUSH_OVERLOAD = 0x05
    FIX_DUST_ENGINE = 0x06
    FIX_FRONT_PANEL = 0x07
    FIX_RADAR_MASK = 0x08
    FIX_DROP_SENSOR = 0x09
    FIX_LOW_BATTERY = 0x0A
    FIX_ABNORMAL_POSTURE = 0x0B
    FIX_LASER_SENSOR = 0x0C
    FIX_EDGE_SENSOR = 0x0D
    FIX_START_IN_FORBID_AREA = 0x0E
    FIX_START_IN_STRONG_MAGNETIC = 0x0F
    FIX_LASER_SENSOR_BLOCKED = 0x10


class B8ErrorRebootDescription(IntEnum):
    """Midea B8 error reboot description."""

    NO = 0x00
    REBOOT_LASER_COMM_FAIL = 0x01
    REBOOT_ROBOT_COMM_FAIL = 0x02
    REBOOT_INNER_FAIL = 0x03


class B8ErrorWarningDescription(IntEnum):
    """Midea B8 error warning description."""

    NO = 0x00
    WARN_LOCATION_FAIL = 0x01
    WARN_LOW_BATTERY = 0x02
    WARN_FULL_DUST = 0x03
    WARN_LOW_WATER = 0x04


class B8StatusType(IntEnum):
    """B8 Status Type."""

    X01 = 0x01
